@@ -88,8 +88,11 @@ void Acceptor::init(
 
     if (!sslCtxManager_) {
       sslCtxManager_ = std::make_unique<SSLContextManager>(
-          "vip_" + getName(), accConfig_.strictSSL, stats);
+          "vip_" + getName(),
+          SSLContextManagerSettings().setStrict(accConfig_.strictSSL),
+          stats);
     }
+    getFizzPeeker()->setSSLContextManager(sslCtxManager_);
     try {
       // If the default ctx is nullptr, we can assume it hasn't been configured
       // yet.
@@ -102,6 +105,15 @@ void Acceptor::init(
               accConfig_.bindAddress,
               cacheProvider_);
         }
+        for (const auto& sniConfig : accConfig_.sniConfigs) {
+          sslCtxManager_->addSSLContextConfig(
+              sniConfig.snis,
+              sniConfig.contextConfig,
+              accConfig_.sslCacheOptions,
+              &accConfig_.initialTicketSeeds,
+              accConfig_.bindAddress,
+              cacheProvider_);
+        }
       }
       CHECK(sslCtxManager_->getDefaultSSLCtx());
     } catch (const std::runtime_error& ex) {
@@ -208,11 +220,13 @@ void Acceptor::resetSSLContextConfigs(
     } else if (sslCtxManager_) {
       sslCtxManager_->resetSSLContextConfigs(
           accConfig_.sslContextConfigs,
+          accConfig_.sniConfigs,
           accConfig_.sslCacheOptions,
           nullptr,
           accConfig_.bindAddress,
           cacheProvider_);
     }
+    getFizzPeeker()->setSSLContextManager(sslCtxManager_);
   } catch (const std::runtime_error& ex) {
     LOG(ERROR) << "Failed to re-configure TLS: " << ex.what()
                << "will keep old config";
@@ -235,7 +249,7 @@ void Acceptor::setTLSTicketSecrets(
   }
 }
 
-void Acceptor::drainAllConnections() {
+void Acceptor::startDrainingAllConnections() {
   if (downstreamConnectionManager_) {
     downstreamConnectionManager_->initiateGracefulShutdown(
         gracefulShutdownTimeout_);
@@ -261,7 +275,7 @@ void Acceptor::acceptConnection(
     folly::NetworkSocket fdNetworkSocket,
     const SocketAddress& clientAddr,
     AcceptInfo info,
-    folly::AsyncTransport::LifecycleObserver* observer) noexcept {
+    folly::AsyncSocket::LegacyLifecycleObserver* observer) noexcept {
   int fd = fdNetworkSocket.toFd();
 
   namespace fsp = folly::portability::sockets;
@@ -288,7 +302,7 @@ void Acceptor::onDoneAcceptingConnection(
     const SocketAddress& clientAddr,
     std::chrono::steady_clock::time_point acceptTime,
     const AcceptInfo& info,
-    folly::AsyncTransport::LifecycleObserver* observer) noexcept {
+    folly::AsyncSocket::LegacyLifecycleObserver* observer) noexcept {
   TransportInfo tinfo;
   tinfo.timeBeforeEnqueue = info.timeBeforeEnqueue;
   processEstablishedConnection(fd, clientAddr, acceptTime, tinfo, observer);
@@ -299,7 +313,7 @@ void Acceptor::processEstablishedConnection(
     const SocketAddress& clientAddr,
     std::chrono::steady_clock::time_point acceptTime,
     TransportInfo& tinfo,
-    folly::AsyncTransport::LifecycleObserver* observer) noexcept {
+    folly::AsyncSocket::LegacyLifecycleObserver* observer) noexcept {
   bool shouldDoSSL = false;
   if (accConfig_.isSSL()) {
     CHECK(sslCtxManager_);
@@ -362,12 +376,12 @@ static std::string logContext(folly::AsyncTransport& transport) {
   int socketFd = -1;
   try {
     localAddr = transport.getLocalAddress().describe();
-  } catch (folly::AsyncSocketException& ex) {
+  } catch (folly::AsyncSocketException&) {
     localAddr = "(unknown)";
   }
   try {
     remoteAddr = transport.getPeerAddress().describe();
-  } catch (folly::AsyncSocketException& ex) {
+  } catch (folly::AsyncSocketException&) {
     remoteAddr = "(unknown)";
   }
 
@@ -381,28 +395,58 @@ static std::string logContext(folly::AsyncTransport& transport) {
 AsyncTransport::UniquePtr Acceptor::transformTransport(
     AsyncTransport::UniquePtr sock) {
   if constexpr (fizz::platformCapableOfKTLS) {
+    fizz::KTLSRxPad rxPad = accConfig_.fizzConfig.expectNoPadKTLSRx
+        ? fizz::KTLSRxPad::RxExpectNoPad
+        : fizz::KTLSRxPad::RxPadUnknown;
     if (accConfig_.fizzConfig.preferKTLS) {
-      std::string sockLogContext;
-      if (VLOG_IS_ON(5)) {
-        sockLogContext = logContext(*sock);
-      }
+      if (accConfig_.fizzConfig.preferKTLSRx) {
+        std::string sockLogContext;
+        if (VLOG_IS_ON(5)) {
+          sockLogContext = logContext(*sock);
+        }
 
-      auto fizzSocket =
-          sock->getUnderlyingTransport<fizz::server::AsyncFizzServer>();
-      if (!fizzSocket) {
-        VLOG(5) << "Acceptor configured to prefer kTLS, but peer is not fizz. "
-                << sockLogContext;
-        return sock;
-      }
-      auto ktlsSockResult = fizz::tryConvertKTLS(*fizzSocket);
-      if (ktlsSockResult.hasValue()) {
-        VLOG(5) << "Upgraded socket to kTLS. " << sockLogContext;
-        return std::move(ktlsSockResult).value();
+        auto fizzSocket =
+            sock->getUnderlyingTransport<fizz::server::AsyncFizzServer>();
+        if (!fizzSocket) {
+          VLOG(5)
+              << "Acceptor configured to prefer kTLS Rx, but peer is not fizz. "
+              << sockLogContext;
+          return sock;
+        }
+        auto ktlsRxSockResult = fizz::tryConvertKTLSRx(*fizzSocket, rxPad);
+        if (ktlsRxSockResult.hasValue()) {
+          VLOG(5) << "Upgraded socket to kTLS Rx. " << sockLogContext;
+          return std::move(ktlsRxSockResult).value();
+        } else {
+          VLOG(5) << "Failed to upgrade to kTLS Rx. ex="
+                  << folly::exceptionStr(ktlsRxSockResult.error()) << " "
+                  << sockLogContext;
+          return sock;
+        }
       } else {
-        VLOG(5) << "Failed to upgrade to kTLS. ex="
-                << folly::exceptionStr(ktlsSockResult.error()) << " "
-                << sockLogContext;
-        return sock;
+        std::string sockLogContext;
+        if (VLOG_IS_ON(5)) {
+          sockLogContext = logContext(*sock);
+        }
+
+        auto fizzSocket =
+            sock->getUnderlyingTransport<fizz::server::AsyncFizzServer>();
+        if (!fizzSocket) {
+          VLOG(5)
+              << "Acceptor configured to prefer kTLS, but peer is not fizz. "
+              << sockLogContext;
+          return sock;
+        }
+        auto ktlsSockResult = fizz::tryConvertKTLS(*fizzSocket, rxPad);
+        if (ktlsSockResult.hasValue()) {
+          VLOG(5) << "Upgraded socket to kTLS. " << sockLogContext;
+          return std::move(ktlsSockResult).value();
+        } else {
+          VLOG(5) << "Failed to upgrade to kTLS. ex="
+                  << folly::exceptionStr(ktlsSockResult.error()) << " "
+                  << sockLogContext;
+          return sock;
+        }
       }
     }
   }
@@ -463,7 +507,7 @@ void Acceptor::sslConnectionReady(
   connectionReady(
       std::move(sock), clientAddr, nextProtocol, secureTransportType, tinfo);
   if (state_ == State::kDraining) {
-    checkDrained();
+    checkIfDrained();
   }
 }
 
@@ -471,7 +515,7 @@ void Acceptor::sslConnectionError(const folly::exception_wrapper&) {
   CHECK(numPendingSSLConns_ > 0);
   --numPendingSSLConns_;
   if (state_ == State::kDraining) {
-    checkDrained();
+    checkIfDrained();
   }
 }
 
@@ -487,30 +531,32 @@ void Acceptor::acceptError(const std::exception& ex) noexcept {
 void Acceptor::acceptStopped() noexcept {
   VLOG(3) << "Acceptor " << this << " acceptStopped()";
   // Drain the open client connections
-  drainAllConnections();
+  startDrainingAllConnections();
 
   // If we haven't yet finished draining, begin doing so by marking ourselves
-  // as in the draining state. We must be sure to hit checkDrained() here, as
+  // as in the draining state. We must be sure to hit checkIfDrained() here, as
   // if we're completely idle, we can should consider ourself drained
   // immediately (as there is no outstanding work to complete to cause us to
   // re-evaluate this).
   if (state_ != State::kDone) {
     state_ = State::kDraining;
-    checkDrained();
+    checkIfDrained();
   }
 }
 
 void Acceptor::onEmpty(const ConnectionManager&) {
   VLOG(3) << "Acceptor=" << this << " onEmpty()";
   if (state_ == State::kDraining) {
-    checkDrained();
+    checkIfDrained();
   }
 }
 
-void Acceptor::checkDrained() {
+void Acceptor::checkIfDrained() {
   CHECK(state_ == State::kDraining);
   if (forceShutdownInProgress_ ||
-      (downstreamConnectionManager_->getNumConnections() != 0) ||
+      (downstreamConnectionManager_ &&
+       downstreamConnectionManager_->getNumConnections() != 0) ||
+
       (numPendingSSLConns_ != 0)) {
     return;
   }
@@ -519,10 +565,7 @@ void Acceptor::checkDrained() {
           << base_;
 
   downstreamConnectionManager_.reset();
-
-  state_ = State::kDone;
-
-  onConnectionsDrained();
+  transitionToDrained();
 }
 
 void Acceptor::drainConnections(double pctToDrain) {
@@ -561,8 +604,7 @@ void Acceptor::dropAllConnections() {
   }
   CHECK(numPendingSSLConns_ == 0);
 
-  state_ = State::kDone;
-  onConnectionsDrained();
+  transitionToDrained();
 }
 
 void Acceptor::dropConnections(double pctToDrop) {
@@ -582,7 +624,7 @@ void Acceptor::dropConnections(double pctToDrop) {
 void Acceptor::dropEstablishedConnections(
     double pctToDrop,
     const std::function<bool(ManagedConnection*)>& filter) {
-  base_->runInEventBaseThread([&, pctToDrop, filter] {
+  base_->runInEventBaseThread([this, pctToDrop, filter] {
     if (downstreamConnectionManager_) {
       VLOG(3) << "Dropping " << pctToDrop * 100 << "% of "
               << getNumConnections()
@@ -596,6 +638,22 @@ void Acceptor::dropEstablishedConnections(
   });
 }
 
+void Acceptor::dropIdleConnectionsBasedOnTimeout(
+    std::chrono::milliseconds targetIdleTimeMs,
+    const std::function<void(size_t)>& droppedConnectionsCB) {
+  base_->runInEventBaseThread([this, targetIdleTimeMs, droppedConnectionsCB] {
+    if (downstreamConnectionManager_) {
+      VLOG(3) << "Dropping connections based on idle timeout "
+              << targetIdleTimeMs.count() << " from acceptor=" << this
+              << " in thread " << base_;
+      assert(base_->isInEventBaseThread());
+
+      downstreamConnectionManager_->dropIdleConnectionsBasedOnTimeout(
+          targetIdleTimeMs, droppedConnectionsCB);
+    }
+  });
+}
+
 Acceptor::AcceptObserverList::AcceptObserverList(Acceptor* acceptor)
     : acceptor_(acceptor) {}
 
