@@ -192,7 +192,7 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
   }
 
   /**
-   * Time after drainAllConnections() or acceptStopped() during which
+   * Time after startDrainingAllConnections() or acceptStopped() during which
    * new requests on connections owned by the downstream
    * ConnectionManager will be processed normally.
    */
@@ -230,7 +230,7 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
       const folly::SocketAddress& clientAddr,
       std::chrono::steady_clock::time_point acceptTime,
       const AcceptInfo& info,
-      folly::AsyncTransport::LifecycleObserver* observer) noexcept;
+      folly::AsyncSocket::LegacyLifecycleObserver* observer) noexcept;
 
   /**
    * Begins either processing HTTP bytes (HTTP) or the SSL handshake (HTTPS)
@@ -240,7 +240,7 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
       const folly::SocketAddress& clientAddr,
       std::chrono::steady_clock::time_point acceptTime,
       TransportInfo& tinfo,
-      folly::AsyncTransport::LifecycleObserver* observer = nullptr) noexcept;
+      folly::AsyncSocket::LegacyLifecycleObserver* observer = nullptr) noexcept;
 
   /**
    * Creates and starts the handshake manager.
@@ -253,10 +253,11 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
       TransportInfo& tinfo) noexcept;
 
   /**
-   * Drains all open connections of their outstanding transactions. When
-   * a connection's transaction count reaches zero, the connection closes.
+   * Starts draining all open connections of their outstanding transactions
+   * asynchronously. When a connection's transaction count reaches zero, the
+   * connection closes.
    */
-  virtual void drainAllConnections();
+  virtual void startDrainingAllConnections();
 
   /**
    * Drain defined percentage of connections.
@@ -288,6 +289,18 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
       double pctToDrop,
       const std::function<bool(ManagedConnection*)>& filter);
 
+  /**
+   * Drops every idle for which idle time is less then *timeout*
+   *
+   * @param timeout - idle time threshold.
+   * @param droppedConnectionsCB - Callback will be invoked for dropped
+   *   connections, having number of dropped connection as input.
+   *
+   */
+  virtual void dropIdleConnectionsBasedOnTimeout(
+      std::chrono::milliseconds targetIdleTimeMs,
+      const std::function<void(size_t)>& droppedConnectionsCB);
+
   /**
    * Wrapper for connectionReady() that can be overridden by
    * subclasses to deal with plaintext connections.
@@ -465,7 +478,7 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
       folly::NetworkSocket fdNetworkSocket,
       const folly::SocketAddress& clientAddr,
       AcceptInfo /* info */,
-      folly::AsyncTransport::LifecycleObserver* lifeCycleObserver) noexcept;
+      folly::AsyncSocket::LegacyLifecycleObserver* lifeCycleObserver) noexcept;
 
  protected:
   // TODO(T81599451): Remove the 'using' statement below after
@@ -527,6 +540,11 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
   folly::AsyncTransport::UniquePtr transformTransport(
       folly::AsyncTransport::UniquePtr sock);
 
+  void transitionToDrained() {
+    state_ = State::kDone;
+    onConnectionsDrained();
+  }
+
   TLSTicketKeySeeds ticketSecrets_;
   std::shared_ptr<fizz::server::CertManager> fizzCertManager_{nullptr};
 
@@ -534,7 +552,7 @@ class Acceptor : public folly::AsyncServerSocket::AcceptCallback,
   Acceptor(Acceptor const&) = delete;
   Acceptor& operator=(Acceptor const&) = delete;
 
-  void checkDrained();
+  void checkIfDrained();
 
   State state_{State::kInit};
   uint64_t numPendingSSLConns_{0};
